<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\DeleteCascadeService;
use Service\HierarchyType as HierarchyTypeService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Util\Utilities;

class HierarchyType extends Crud{
    
    protected $dataSourceName = 'hierarchyType';
    protected $hierarchyType;
    
    use TResponse;    
    
    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\HierarchyType $hierarchyType) {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->hierarchyType = $hierarchyType;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }
   
    public function findNext(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $hierarchyTypes = $this->hierarchyType->findHierarchy($params);
        $strucuresTypes = $this->hierarchyType->getStrucutresTypes($hierarchyTypes)->getRows();
        $indexedStructureTypes = $this->hierarchyType->indexByMongoId($strucuresTypes);
        $hierarchyTypes = $this->relateHierarchyWithStructureType($hierarchyTypes, $indexedStructureTypes);
        $indexedHierarchyTypes = $this->hierarchyType->indexByMongoId($hierarchyTypes);
        $hierarchyTypes = $this->relateHierarchyWithHierarchyParent($hierarchyTypes, $indexedHierarchyTypes);
        $this->sendResponse(Utilities::parseDataSourceZeedhiNext($hierarchyTypes), 200);
    }

    protected function relateHierarchyWithStructureType($hierarchyTypes, $indexedByIdStructureTypes) {
        foreach($hierarchyTypes as &$hierarchyType) {
            $hierarchyType['typeParentName'] = $indexedByIdStructureTypes[$hierarchyType['typeParent']]['name'];
            $hierarchyType['structureTypeParent'] =  $indexedByIdStructureTypes[$hierarchyType['typeParent']];
            $hierarchyType['typeChildName'] = $indexedByIdStructureTypes[$hierarchyType['typeChild']]['name'];
            $hierarchyType['structureTypeChild'] =  $indexedByIdStructureTypes[$hierarchyType['typeChild']];
        }
        return $hierarchyTypes;
    }

    protected function relateHierarchyWithHierarchyParent($hierarchyTypes, $indexedByIdHierarchyTypes) {
        foreach($hierarchyTypes as &$hierarchyType) {
            if (isset($hierarchyType['parent'])) {
                $hierarchyType['parentName'] = $indexedByIdHierarchyTypes[$hierarchyType['parent']]['name'];
            }
        }
        return $hierarchyTypes;
    }
}