<?php

namespace Service;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Driver\OCI8\OCI8Exception;
use Doctrine\DBAL\Driver\OCI8\OCI8Statement;

class ColumnsQueryService {

    /** @var Connection */
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function getQueryResult($query) {
        return $this->connection->fetchAll($query);
    }

    /**
     * @param $query
     * @return array
     * @throws OCI8Exception
     * @throws \Exception
     */
    public  function getColumnsOfAQuery($query) {
        // @todo Fix implementing getColumnMetaData in \Doctrine\DBAL\Statement.
        $columnNames = array();

        $stmt = $this->connection->getWrappedConnection()->prepare($query);
        switch (get_class($stmt)) {
            case 'Doctrine\DBAL\Driver\OCI8\OCI8Statement':
            case 'Zeedhi\Framework\DBAL\Driver\OCI8\OCI8Statement':
                $columnNames = $this->getColumnsOfAQueryForOCI8($stmt);
            break;
            default:
                throw new \Exception("UNSUPPORTED_DRIVER");
            break;
        }

        return $columnNames;
    }

    protected function getColumnsOfAQueryForOCI8(OCI8Statement $stmt) {
        $columnNames = array();
        $reflection = new \ReflectionProperty('Doctrine\DBAL\Driver\OCI8\OCI8Statement', "_sth");
        $reflection->setAccessible(true);
        $ociStmt = $reflection->getValue($stmt);
        $ret = @oci_execute($ociStmt, OCI_DESCRIBE_ONLY);
        if (!$ret) {
            throw OCI8Exception::fromErrorInfo(oci_error($ociStmt));
        }
        $nrCols = oci_num_fields($ociStmt);
        for ($i = 1; $i <= $nrCols; $i++) {
            $columnNames[] = oci_field_name($ociStmt, $i);
        }
        return $columnNames;
    }
}