<?php

namespace Service;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Service\Manager\Mongo\Manager;

class HierarchyType {
	
    protected $dataSourceName = 'hierarchyType';
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $organizationManager;


	public function __construct(Manager $organizationManager)  {
        $this->organizationManager = $organizationManager;
	}

    public function findBy(FilterCriteria $filterCriteria) {
        return $this->organizationManager->findBy($filterCriteria)->getRows();
    }

    public function findById($id) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(array("columnName"=>"_id", "value" => $id, "operator" => "=")), null, 10000);
        $structure = $this->organizationManager->findBy($filterCriteria)->getRows();
        return count($structure) > 0 ? $structure[0] : null;;
    }

    public function findAll() {
        return $this->organizationManager->findBy(new FilterCriteria($this->dataSourceName, array(), null, 10000))->getRows();
    }

    public function findHierarchy($params) {
        $response = $this->organizationManager->findNext($this->dataSourceName, $params);
        return $response;
    }
    
    public function getStrucutresTypes ($hierarchyTypes) {
        $usedIds = $this->getStructureIdsUsed($hierarchyTypes);
        $numberOfExpectedLines = count($usedIds);
        $filterCriteria = new FilterCriteria('structureType', array(), null, $numberOfExpectedLines);
        $filterCriteria->addCondition('_id', FilterCriteria::IN, $usedIds);
        return $this->organizationManager->findBy($filterCriteria);
    }
    
    public function getStructureIdsUsed ($hierarchyTypes) {
        //For not repeat Ids used more than one time
        $structuresIdsIndexedById = $this->getDistinctStructureIds($hierarchyTypes);
        //Build distinct ids in a index array
        $distictStruturesIds = array();
        foreach($structuresIdsIndexedById as $structureId) {
            $distictStruturesIds[] = $structureId;
        }
        return $distictStruturesIds;
    }
    
    public function getDistinctStructureIds($hierarchyTypes) {
        $structuresIdsIndexedById = array();
        foreach($hierarchyTypes as $hierarchyType) {
            $parentTypeId = $hierarchyType['typeParent'];
            $structuresIdsIndexedById[$parentTypeId] = $parentTypeId;
            $childTypeId = $hierarchyType['typeChild'];
            $structuresIdsIndexedById[$childTypeId] = $childTypeId;
        }
        return $structuresIdsIndexedById;
    }
    
    public function indexByMongoId($rows) {
        $newRows = array();
        foreach($rows as $row) {
            $newRows[$row['_id']] = $row;
        }
        return $newRows;
    }

    public function getChildStructureTypeByStructureType($structureType) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(), null, 10000);
        $filterCriteria->addCondition('typeParent', $structureType);
        $hierarchyType = $this->organizationManager->findBy($filterCriteria)->getRows();
        $childStructureType = null;
        if (!empty($hierarchyType)) {
            $hierarchyType = $hierarchyType[0];
            $filterCriteria = new FilterCriteria('structureType', array(), null, 10000);
            $filterCriteria->addCondition(Manager::MONGO_KEY_COLUMN, $hierarchyType['typeChild']);
            $childStructureType = $this->organizationManager->findBy($filterCriteria)->getRows();
            $childStructureType = empty($childStructureType)? null : $childStructureType[0];
        }

        return $childStructureType;
    }

}