<?php


namespace Service\PeriodMathExpression;


class Period
{

    /**
     * @param string $expression
     * @return string
     * @throws \Exception
     */
    public static function getDate($expression = ''){

        $date = self::decode($expression);
        return $date->format('d/m/Y');
    }



    /**
     * @param $expression
     * @return array
     */
    protected static function getArrayExpression($expression)
    {
        $length = strlen($expression);
        $arrayExpression = array();
        $singleExpression = '';
        for ($i = 0; $i < $length; $i++) {
            $char = $expression[$i];
            if (in_array($char, Strategy\UnitFactory::OPERATIONS)) {
                $arrayExpression[] = $singleExpression;
                $singleExpression = '';
            }
            $singleExpression .= $char;

        }
        $arrayExpression[] = $singleExpression;
        return $arrayExpression;
    }

    public static function getDateMoment($expression = '',$dateBase=''){
        $dateTime = self::decode($expression, $dateBase);
        return \Util\Period::convertDateTimeToMoment($dateTime);
    }

    /**
     * @param $expression
     * @param string $dateBase
     * @return \DateTime
     * @throws \Exception
     */
    protected static function decode($expression, $dateBase = ''): \DateTime
    {
        $date = new \DateTime();
        if(!empty($dateBase)){
            $date = \DateTime::createFromFormat('Y-m-d H:i:s', $dateBase.' 00:00:00');
        }
        $arrayExpression = self::getArrayExpression($expression);
        foreach ($arrayExpression as $dateExpression) {
            if ($dateExpression) {
                $operator = $dateExpression[0];
                $unit = $dateExpression[strlen($dateExpression) - 1];
                $quantity = preg_replace("/[^0-9]/", "", $dateExpression);
                if (!$quantity && $quantity != '0') {
                    throw New \Exception('QUANTITY_NOT_FOUND');
                }
                $strategy = Strategy\UnitFactory::factory($unit);
                $date = $strategy->modify($date, $operator, $quantity);
            }
        }
        return $date;
    }
}