<?php

namespace Util;

use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\Mongo\Mongo;
use Moment\Moment;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;

class MongoDataBaseAlterations {

    protected $mongoParams;
    protected $mongoParamsIdentity;
    protected $mongoManager;
    protected $mongoManagerIdentity;
    public $ORG_DB_PREFIX = 'Zmart__';
    public $ORG_DB_PREFIX_OLD = 'Zmart_';

    // protected $ORGS = array(100, 101, 102, 1051, 1136, 1221, 1226, 1321, 1417,
    //     1437, 1457, 1477, 1478, 1497, 1517, 1537, 1557, 1577, 1597, 1617, 1639,
    //     1657, 1677, 1697, 1717, 1737, 1778, 1797, 1817, 1837, 1857, 1877, 1897,
    //     1898, 1917, 1957, 1977, 1997, 2017, 2037, 2038, 2057, 206, 2081, 2082,
    //     2083, 2084, 2085, 211, 2110, 2111, 2131, 2132, 2138, 2154, 2155, 2156,
    //     2157, 2159, 216, 2179, 2200, 2201, 2202, 2222, 2242, 2262, 2263, 2264,
    //     2284, 2285, 2286, 2307, 2326, 2327, 2346, 2366, 2367, 2407, 2408, 2428,
    //     2429, 2449, 2469, 2470, 2471, 2472, 2473, 261, 316, 621, 801, 821, 826);
    protected $ORGS = array();
    // $relates  = array( old , new )
    protected $relatedIds = array();
    protected $relatedIdUsers = array();
    protected $relatedAccessProfiles = array();

    public function __construct($mongoParams,$mongoParamsIdentity = null) {
        $this->mongoParams = $mongoParams;
        $this->mongoParamsIdentity  = $mongoParamsIdentity ;
        $this->mongoManager = $this->buildManager('Zmart');
        if($this->mongoParamsIdentity){
            $this->mongoManagerIdentity = $this->buildManager('Zmart');
        }
    }

    public function setOrgs($org){
        $this->ORGS = array($org);
    }

    public function setRelatedIdUsers($relatedIdUsers){
        $this->relatedIdUsers = $relatedIdUsers;
    }


    public function setRelatedIds($relatedIds){
        $this->relatedIds = $relatedIds;
    }

    public function setRelatedAccessProfiles($relatedAccessProfiles){
        $this->relatedAccessProfiles = $relatedAccessProfiles;
    }

    /** Method to generate a new mongo manager **/
    public function buildManager($dbName){
        $server = $this->mongoParams['server'];
        $port = $this->mongoParams['port'];
        $mongoOptions = $this->mongoParams['mongoOptions'];

        $im = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();

        $mongo = new Mongo($server, $port, $dbName, $mongoOptions);
        $newManager = new Manager($mongo, $im->getService("nameProvider"), $im->getService("mongoQueryBuilder"));

        return $newManager;
    }

    /** Method to get files at mongo collection **/
    public function findDocuments($dataSourceName, $criterias = array(), $mongoManager = null) {
        $mongoManager = $mongoManager ? $mongoManager : $this->mongoManager;
        $filterCriteria = new FilterCriteria($dataSourceName);

        foreach ($criterias as $key => $value) {
            $filterCriteria->addCondition($key, $value);
        }

        $results = $mongoManager->findBy($filterCriteria)->getRows();
        return $results;
    }

    /** Method to change files at mongo collection **/
    public function changeFiles($dataSourceName, $items, $mongoManager = null) {
        $mongoManager = $mongoManager ? $mongoManager : $this->mongoManager;
        $dataSet = new DataSet($dataSourceName, $items);
        return $mongoManager->persist($dataSet);
    }

    /*Procura os ids das organizações dentro de cada organização*/
    public function changeOrganizationIdOfImoportHistory($organization, $newOrganization) {
        $dbName = $this->ORG_DB_PREFIX.$newOrganization;
        $orgMongoManager = $this->buildManager($dbName);

        $collections = $this->findDocuments('importHistory', array(), $orgMongoManager);

        foreach ($collections as &$collection) {
            $collection["organizationId"] = $newOrganization;
        }

        $this->changeFiles('importHistory', $collections, $orgMongoManager);
    }

    public function changeOrganizationIdOfConnection($organization, $newOrganization) {
        $dbName = $this->ORG_DB_PREFIX.$newOrganization;
        $orgMongoManager = $this->buildManager($dbName);

        $collections = $this->findDocuments('connection', array(), $orgMongoManager);

        foreach ($collections as &$collection) {
            $collection["organizationId"] = $newOrganization;
        }

        $this->changeFiles('connection', $collections, $orgMongoManager);
    }

    public function deleteAllImportScheduleBiByOrganization($newOrganization){
        $criterias = array(
            'organizationId' => ''.$newOrganization,
            'type' => 'scheduledBI',
        );
        $filterCriteria = new FilterCriteria('importSchedule');

        foreach ($criterias as $key => $value) {
            $filterCriteria->addCondition($key, $value);
        }

        $importSchedule = $this->mongoManager->findBy($filterCriteria);
        $this->mongoManager->delete($importSchedule);
    }

    /*Procura os ids das organizações dentro da orzanização principal Zmart */
    public function changeOrganizationIdOfImportShedulePrimary($organization, $newOrganization){

        $this->deleteAllImportScheduleBiByOrganization($newOrganization);

        $criteria = array( 'organizationId' => ''.$organization.'');
        $importSchedule = $this->findDocuments("importSchedule", $criteria,$this->mongoManagerIdentity);

        foreach($importSchedule as &$import) {
            $import["organizationId"] = ''.$newOrganization;
            unset($import["organization"]);
            unset($import["_id"]);
            $import['date'] = Type::getType('mongoDate')->convertToDatabaseValue($import['date']);
            if(isset($import['type'])){
                if($import['type'] == 'scheduled'){
                    $import['type'] = 'scheduledBI';
                }else if($import['type'] == 'manual'){
                    $import['type'] = 'manualBI';
                }
            }
        }
        $this->changeFiles('importSchedule', $importSchedule);

    }

    /*Procura os ids das organizações dentro da orzanização principal Zmart */
    public function changeOrganizationIdOfImportShedule($organization, $newOrganization){

        $this->deleteAllImportScheduleBiByOrganization($newOrganization);

        $criteria = array( 'organization' => ''.$organization.'');
        $importSchedule = $this->findDocuments("importSchedule", $criteria,$this->mongoManagerIdentity);

        foreach($importSchedule as &$import) {
            $import["organizationId"] = ''.$newOrganization;
            unset($import["organization"]);
            unset($import["_id"]);
            $import['date'] = Type::getType('mongoDate')->convertToDatabaseValue($import['date']);
            if(isset($import['type'])){
                if($import['type'] == 'scheduled'){
                    $import['type'] = 'scheduledBI';
                }else if($import['type'] == 'manual'){
                    $import['type'] = 'manualBI';
                }
            }
        }
        $this->changeFiles('importSchedule', $importSchedule);

    }

    public function createOrganizationIdOfConnectionInsideZmart($organization, $newOrganization){
        $criteria = array( 'organizationId' => ''.$organization.'');
        $connections = $this->findDocuments("connection", $criteria);

        foreach($connections as &$connection) {
            unset($connection["_id"]);
            $connection["organizationId"] = $newOrganization;
            if(isset($connection["clientRoom"]) && $connection["clientRoom"] == $organization){
                unset($connection["clientRoom"]);
            }
        }

        $this->changeFiles('connection', $connections);
    }

    public function changeOrganizationIdOfConnectionInsideZmart($organization, $newOrganization){
        $criteria = array( 'organization' => ''.$organization.'');
        $connections = $this->findDocuments("connection", $criteria);

        foreach($connections as &$connection) {
            $connection["organizationId"] = $newOrganization;
            if(isset($connection["clientRoom"]) && $connection["clientRoom"] == $organization){
                unset($connection["clientRoom"]);
            }
        }

        $this->changeFiles('connection', $connections);
    }

    public function changeOrganizationIdOfDependentDataSources($organization, $newOrganization){
        $criteria = array( 'organization' => $organization);
        $connections = $this->findDocuments("dependentDataSources", $criteria,$this->mongoManagerIdentity);

        foreach($connections as &$connection) {
            $connection["organizationId"] = $newOrganization;
        }

        $this->changeFiles('dependentDataSources', $connections);
    }

    public function changeOrganizationIdOfDependentDataSourcesPrimary($organization, $newOrganization)
    {
        $criteria = array('organizationId' => $organization);
        $connections = $this->findDocuments("dependentDataSources", $criteria);

        foreach ($connections as &$connection) {
            unset($connection["_id"]);
            $connection["organizationId"] = $newOrganization;
        }

        $this->changeFiles('dependentDataSources', $connections);
    }

    /*função que muda os ids das organizações*/
    public function changeOrganizationIdsPrimary (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $this->createOrganizationIdOfConnectionInsideZmart($idOrg, $nrOrg);
            $this->changeOrganizationIdOfImportShedulePrimary($idOrg, $nrOrg);
            $this->changeOrganizationIdOfImoportHistory($idOrg, $nrOrg);
            $this->changeOrganizationIdOfDependentDataSourcesPrimary($idOrg, $nrOrg);
            $this->changeOrganizationIdOfConnection($idOrg, $nrOrg);
        }
    }


    /*função que muda os ids das organizações*/
    public function changeOrganizationIds (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $this->changeOrganizationIdOfConnectionInsideZmart($idOrg, $nrOrg);
            $this->changeOrganizationIdOfImportShedule($idOrg, $nrOrg);
            $this->changeOrganizationIdOfImoportHistory($idOrg, $nrOrg);
            $this->changeOrganizationIdOfDependentDataSources($idOrg, $nrOrg);
            $this->changeOrganizationIdOfConnection($idOrg, $nrOrg);
        }
    }

    /*função que muda os ids das organizações*/
    public function changeOrganizationIdsInImportShedule (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $this->changeOrganizationIdOfImportShedule($idOrg, $nrOrg);
        }
    }

    public function addSslInConnection($dbName)
    {
        $orgMongoManager = $this->buildManager($dbName);
        $collections = $this->findDocuments('connection', array(), $orgMongoManager);
        foreach ($collections as &$collection) {
            if (empty($collection["dbUrlSsl"]) && isset($collection["dbUrl"])) {
                $collection["dbUrlSsl"] = OpensslCrypt::encrypt(RijndaelCrypt::decrypt($collection["dbUrl"]));
            }
            if (empty($collection["usernameSsl"]) && isset($collection["username"])) {
                $collection["usernameSsl"] = OpensslCrypt::encrypt(RijndaelCrypt::decrypt($collection["username"]));
            }
            if (empty($collection["passwordSsl"]) && isset($collection["password"])) {
                $collection["passwordSsl"] = OpensslCrypt::encrypt(RijndaelCrypt::decrypt($collection["password"]));
            }
        }
        $this->changeFiles('connection', $collections, $orgMongoManager);
    }

    /*funções que mudam os ids dos usuários dentro de cada organização*/
    public function changeAccesProfilesOfViews() {
        foreach ($this->ORGS as $orgId) {
            $dbName = $this->ORG_DB_PREFIX.$orgId;
            $orgMongoManager = $this->buildManager($dbName);
            $collections = $this->findDocuments('view', array(), $orgMongoManager);

            foreach ($collections as &$collection) {
                if (!empty($collection["accessProfiles"])) {
                    $accessProfiles = array();
                    foreach ($collection["accessProfiles"] as $accessProfile) {
                        if(!empty($this->relatedAccessProfiles[$accessProfile])){
                            $cdaccessProfile = $this->relatedAccessProfiles[$accessProfile];
                            if ($cdaccessProfile) {
                                $accessProfiles[] = $cdaccessProfile;
                            }
                        }else{
                            $accessProfiles[] = $accessProfile;
                        }
                    }
                    $collection["accessProfiles"] = $accessProfiles;
                }
            }
            $this->changeFiles('view', $collections, $orgMongoManager);
        }
    }

    public function changeUsersIdsOfUserStructures() {
        foreach ($this->ORGS as $orgId) {
            $dbName = $this->ORG_DB_PREFIX.$orgId;
            $orgMongoManager = $this->buildManager($dbName);
            $collections = $this->findDocuments('userStructures', array(), $orgMongoManager);

            foreach ($collections as &$collection) {
                if (!empty($this->relatedIdUsers[$collection["user"]])) {
                    $cdoperador = $this->relatedIdUsers[$collection["user"]];
                    $collection["user"] = $cdoperador;
                }
            }

            $this->changeFiles('userStructures', $collections, $orgMongoManager);
        }
    }

    public function changeUsersIdsOfStructures($organization, $newOrganization) {
        foreach ($this->ORGS as $orgId) {
            $dbName = $this->ORG_DB_PREFIX.$orgId;
            $orgMongoManager = $this->buildManager($dbName);
            $collections = $this->findDocuments('structure', array(), $orgMongoManager);
            foreach ($collections as &$collection) {
                if(!empty($collection["usersResponsible"])){
                    $newUsersResponsible = array();
                    foreach($collection["usersResponsible"] as $usersResponsible){
                        if(!empty($this->relatedIdUsers[$usersResponsible])){
                            $cdoperador = $this->relatedIdUsers[$usersResponsible];
                            if($cdoperador){
                                $newUsersResponsible[] = $cdoperador;
                            }
                        }else{
                            $newUsersResponsible[] = $usersResponsible;
                        }
                    }
                    $collection["usersResponsible"] = $newUsersResponsible;
                }
            }
            $this->changeFiles('structure', $collections, $orgMongoManager);
        }
    }

    /*função que muda os ids dos usuarios*/
    public function changeUsersIds (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $this->changeAccesProfilesOfViews($idOrg, $nrOrg);
            $this->changeUsersIdsOfUserStructures($idOrg, $nrOrg);
            $this->changeUsersIdsOfStructures($idOrg, $nrOrg);
        }
    }

    public function saveMigrations (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $this->saveMigrationsByOrganization($idOrg, $nrOrg);;
        }
    }

    public function saveMigrationsByOrganization($organization){
        $criteria = array( 'organization' => ''.$organization.'');
        $connections = $this->findDocuments("connection", $criteria);

        foreach($connections as &$connection) {
            $connection["migration"] = "ok";
        }

        $this->changeFiles('connection', $connections);
    }

    public function getListDatabases()
    {
        $command = ['listDatabases' => 1];
        $newManager = $this->buildManager('admin');
        return $newManager->command($command);
    }

    public function cloneDBNamePrimary (){
        $start= microtime(true);
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $dbName = $this->ORG_DB_PREFIX.$nrOrg;
            $dbNamePrimary = $this->ORG_DB_PREFIX.$idOrg;
            $options = array();
            $options['socketTimeoutMS'] = "3600000";
            $this->mongoParams['mongoOptions'] = $options;
            $newManager = $this->buildManager('admin');
            $result = $newManager->cloneDBName($dbNamePrimary,$dbName,null);
            if(empty($result['ok'])){
                new \Exception($result['errmsg']);
            }
        }
        $end = microtime(true);
        return $end - $start ;
    }

    public function cloneDBName ($mongoHost){
        $start= microtime(true);
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $dbName = $this->ORG_DB_PREFIX.$nrOrg;
            $dbNameOld = $this->ORG_DB_PREFIX_OLD.$idOrg;
            $options = array();
            $options['socketTimeoutMS'] = "3600000";
            $this->mongoParams['mongoOptions'] = $options;
            $newManager = $this->buildManager('admin');
            $result = $newManager->cloneDBName($dbNameOld,$dbName,$mongoHost);
            if(empty($result['ok'])){
                new \Exception($result['errmsg']);
            }
        }
        $end = microtime(true);
        return $end - $start ;
    }

    public function createDbUser (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $dbName = $this->ORG_DB_PREFIX.$nrOrg;
            $this->mongoManager->createDbUser($dbName);
        }
    }

    public function createDbUserPerNrorg (){
        foreach($this->relatedIds as $idOrg => $nrOrg){
            $dbName = $this->ORG_DB_PREFIX.$nrOrg;
            $newManager = $this->buildManager($dbName);
            $newManager->createDbUser($dbName);
            $newManager = $this->buildManager('admin');
            $newManager->createDbUser($dbName);
        }
    }

}